
 @Analytic Gradients For 2 state Markov process@
/* This procedure calculates analytic gradients.*/
/* It requires the following information as globals 
           nstat number of states. 
           vei 	 2*nstat X 1 vector - each element is the number of variables
		 in either a level or transition equations.
           terms is a 2*nstat X 2 vector, a product of mkvstart. 
		 It tells which variables to use in the data matrix
		 for each level and transition equation.
           
 The beta vector is asssumed to be of the form
			 parameters for level equations in state 1...to n 
			 parameters for transition equations in state 1 to sn
                         std deviations
                         if iy = 1 the nstat -1 parameters for intial values.   */



proc (1)=angrad(beta,y);


local pxa,pxp,p,pp,qq,q,pkim,rho,pcr,ppcr,qcr,qqcr,begz,endz,numvar,sigma,errz,angrd,i,nobs,ultp,ultq,pkimso1,pkimso2, pstart,qstart, pfin,qfin, s1s1, s2s2;

nobs=rows(y);
endz=0;
i=0;
numvar=sumc(vei);
errz=zeros(nobs,nstat);
angrd=zeros(nobs,rows(beta));
{pkim,pxa,pxp}=kimsmth(beta,y);
pkim=pkim~(1-pkim);
@The above was done so it could be handled in a loop. When I finish the n-state smoother it will not be needed.@

do while i<nstat;
	i=i+1;
	begz =1+endz;
	endz=begz+ vei[i,1]-1;

	if ix==1;
		sigma=beta[numvar+i,1];
		else;
		sigma= beta[numvar+1,1];
	endif;

	errz[.,i]=y[.,1]-(y[.,terms[i,1]:terms[i,2]]*beta[begz:endz,1]);
	errz[.,i]=errz[.,i]/(sigma);
angrd[.,begz:endz]=(errz[.,i].*y[.,terms[i,1]:terms[i,2]]).*pkim[.,i]/sigma;

	@Sigma gradients when they differ across states@
	if ix == 1;
		angrd[.,numvar+i]=(errz[.,i]^2-1) .*pkim[.,i] /sigma;
	endif;

endo;

@Calculate the sigma gradients if ix = 0@
if ix == 0;
	angrd[.,numvar+1]=sumc( ( (errz[.,.]^2-ones(rows(errz),nstat)) .*pkim[.,.])' /sigma);
endif;

@angrd=sumc(angrd);@


@Calculate the prob gradients. The modification to n-states should be possible. @
 
pstart = sumc(vei[1:nstat,1])+1;
pfin = pstart +vei[nstat+1,1]-1;
qstart = pfin+1;
qfin =  qstart + vei[nstat+2,1]-1;


s1s1 = cdfn(y[.,terms[nstat+1,1]:terms[nstat+1,2]]*beta[pstart:pfin,1]);
s2s2= cdfn(y[.,terms[nstat+2,1]:terms[nstat+2,2]]*beta[qstart:qfin,1]);

p =(pkim[2:nobs,1].*pxp[1:nobs-1,1])./pxa[2:nobs,1];
pp = (pkim[2:nobs,2].*pxp[1:nobs-1,1])./(1-pxa[2:nobs,1]);
q = (pkim[2:nobs,2].*(1-pxp[1:nobs-1,1]))./(1-pxa[2:nobs,1]);
qq = (pkim[2:nobs,1].*(1-pxp[1:nobs-1,1]))./(pxa[2:nobs,1]);
p=p-pp;
q=q-qq;

p=p.*y[2:nobs,terms[nstat+1,1]:terms[nstat+1,2]].*pdfn(y[2:nobs,terms[nstat+1,1]:terms[nstat+1,2]]*beta[pstart:pfin,1]);

q=q.*y[2:nobs,terms[nstat+2,1]:terms[nstat+2,2]].*pdfn(y[2:nobs,terms[nstat+2,1]:terms[nstat+2,2]]*beta[qstart:qfin,1]);

pcr=0;
qcr=0;
ultp=0;
ultq=0;
 
 if iy == 0;            rho = (1-meanc(s2s2))/(2-meanc(s1s1)-meanc(s2s2));

pkimso1=rho*(s1s1[1,1]*pkim[1,1]/pxa[1,1]+(1-s1s1[1,1])*(1-pkim[1,1])/(1-pxa[1,1]));
pkimso2= 1- pkimso1;

ultp=(pkimso1-rho)/(1-rho);
ultp=ultp/(nobs*(2-meanc(s1s1)-meanc(s2s2)));
ultp=ultp*sumc( pdfn(y[1:nobs,terms[nstat+1,1]:terms[nstat+1,2]]*beta[pstart:pfin,1]) .*y[1:nobs,terms[nstat+1,1]:terms[nstat+1,2]]);


ultq=(pkimso2-(1-rho))/(rho);
ultq=ultq/(nobs*(2-meanc(s1s1)-meanc(s2s2)));
ultq=ultq*sumc( pdfn(y[1:nobs,terms[nstat+2,1]:terms[nstat+2,2]]*beta[qstart:qfin,1]) .*y[1:nobs,terms[nstat+2,1]:terms[nstat+2,2]]);

elseif iy == 1;           rho=cdfn(beta[numvar+2+ix*(nstat-1),1]);

pkimso1 = rho*(s1s1[1,1]*pkim[1,1]/pxa[1,1]+(1-s1s1[1,1])*(1-pkim[1,1])/(1-pxa[1,1]));
pkimso2 = 1-pkimso1;

angrd[numvar+2+ix*(nstat-1),1]=pkimso1*pdfn(beta[numvar+2+ix*(nstat-1),1])/(cdfn(beta[numvar+2+ix*(nstat-1),1])*(1-cdfn(beta[numvar+2+ix*(nstat-1),1])));
angrd[numvar+2+ix*(nstat-1),1]= angrd[numvar+2+ix*(nstat-1),1]-pdfn(beta[numvar+2+ix*(nstat-1),1])/(1-cdfn(beta[numvar+2+ix*(nstat-1),1]));

@For the next two cases the derivative with respect to intial conditions is zero@

elseif iy == 2;         rho = 1.0;
elseif iy == 3;         rho = 0;
endif;

if iy /= 3;
	pcr=(pkim[1,1]*rho)./pxa[1,1];
	ppcr=(pkim[1,2]*rho)./(1-pxa[1,1]);
	pcr=pcr-ppcr;
	pcr=pcr.*y[1,terms[nstat+1,1]:terms[nstat+1,2]].*pdfn(y[1,terms[nstat+1,1]:terms[nstat+1,2]]*beta[pstart:pfin,1]);

	p=pcr|p;
endif;

if iy /= 2;
	qcr= (pkim[1,2]*(1-rho))./(1-pxa[1,1]);
	qqcr= (pkim[1,1]*(1-rho) )./pxa[1,1];
	qcr=qcr-qqcr;
	qcr=qcr.*y[1,terms[nstat+2,1]:terms[nstat+2,2]].*pdfn(y[1,terms[nstat+2,1]:terms[nstat+2,2]]*beta[qstart:qfin,1]);
	q=qcr|q;
endif;


@angrd[pstart:pfin,1]=sumc(p)+ultp;@
@angrd[qstart:qfin,1]=sumc(q)+ultq;@

p[1,.]=ultp'+p[1,.];
q[1,.]=ultq'+q[1,.];
angrd[.,pstart:pfin]=p;
angrd[.,qstart:qfin]=q;


retp(angrd);
endp;
