@ bihist() 		by Simon van Norden 3/5/96 @

/*
bihist() is a GAUSS procedure that generates a bivariate histogram.
The syntax is:
	{x,y,z} = bihist(xdata,nbins);
where
	xdata	is a Tx2 matrix of data.
		Each column contains data on a single variable.
		We want to see their joint histogram.

	nbins	is the number of bins to use in the histogram 
		(same for both variables)

	{x,y,z}	are returned by the procedure.  
		This is the data for the 3-D graph.
		The graph can be recreated by calling 
			surface(x,y,z);

Don't forget to include PGRAPH in your library statement!
*/



proc (3)=bihist(x,nbins);
local v, wxy, nrows, maxx, minx, medianx, stddevx, xbin, z, y, xdum, ydum;


nrows=rows(x);
minx=minc(x)';
maxx=maxc(x)';
medianx=median(x)';
stddevx=stdc(x)';
minx=minx | (medianx - 3.*stddevx);
minx = maxc(minx)';
maxx=maxx | (medianx + 3.*stddevx);
maxx = minc(maxx)';

xdum=dummy(x[.,1],seqa(minx[.,1],(maxx[.,1]-minx[.,1])/nbins,nbins));
ydum=dummy(x[.,2],seqa(minx[.,2],(maxx[.,2]-minx[.,2])/nbins,nbins));
z=100*(xdum'*ydum)./nrows;

x=seqa(minx[.,1],(maxx[.,1]-minx[.,1])/(nbins+1),nbins+1);
y=seqa(minx[.,2],(maxx[.,2]-minx[.,2])/(nbins+1),nbins+1);

zlabel("Frequency (in %)");
surface(x',y,z);

retp(x',y,z);

endp;
