/*
MKVSPEC() by Simon van Norden  29-7-93
Modified by Rob Vigfusson March 95


This procedure uses wmisspec() from wmisspec.g to calculate a series of diagnostic tests for Markov switching regressions.
You must ensure that you have #INCLUDEd wmisspec.g before this file.

When the function is called, it is assumed that mkvstart() has already been run in order to set up the parameters for the Markovian likelihood function.

*/


proc (0)= mkvspec(beta0,dta);
local score, c, a, chistat, signif, nobs, ncol, n1, n2, n3, n4;

"White's Misspecification Tests:";
score = angrad(beta0,dta);		
"Relative Gradient is :";	(sumc(score)./beta0)';
ncol=cols(score);	nobs=rows(dta);

@Take first entry in each equation. Assumed to be a constant @

n1=1; 	n2=vei[1]+1; 	n3=sumc(vei[1:2])+1; 	n4=sumc(vei[1:3])+1;

if ix;
	"Joint Test for Misspecification is Chi-sq(6) under the null.";
	{chistat,signif}=wmisspec(score,(n1|n2|n3|n4|ncol-1|ncol));
	"Test statistic = ";;	chistat;;
	",  Signif. = ";; 	signif;
else;
	"Joint Test for Misspecification is Chi-sq(5) under the null.";
	{chistat,signif}=wmisspec(score,(n1|n2|n3|n4|ncol));
	"Test statistic = ";;	chistat;;
	",  Signif. = ";; 	signif;
endif;

"AR(1) Test for Regime 1 is Chi-Sq(1) under the null.";
{chistat,signif}=wmisspec(score,n1);
"Test statistic = ";;	chistat;;
",  Signif. = ";; 	signif;

"AR(1) Test for Regime 2 is Chi-Sq(1) under the null.";
{chistat,signif}=wmisspec(score,n2);
"Test statistic = ";;	chistat;;
",  Signif. = ";; 	signif;

if ix;
	"ARCH(1) Test for Regime 1 is Chi-Sq(1) under the null.";
	{chistat,signif}=wmisspec(score,ncol-1);
	"Test statistic = ";;	chistat;;
	",  Signif. = ";; 	signif;

	"ARCH(1) Test for Regime 2 is Chi-Sq(1) under the null.";
	{chistat,signif}=wmisspec(score,ncol);
	"Test statistic = ";;	chistat;;
	",  Signif. = ";; 	signif;
else;
	"ARCH(1) Test is Chi-Sq(1) under the null.";
	{chistat,signif}=wmisspec(score,ncol);
	"Test statistic = ";;	chistat;;
	",  Signif. = ";; 	signif;
endif;

"Test for higher-order Markov Effects in Regime 1 is Chi-Sq(1) under the null.";
{chistat,signif}=wmisspec(score,n3);
"Test statistic = ";;	chistat;;
",  Signif. = ";; 	signif;

"Test for higher-order Markov Effects in Regime 2 is Chi-Sq(1) under the null.";
{chistat,signif}=wmisspec(score,n4);
"Test statistic = ";;	chistat;;
",  Signif. = ";; 	signif;

endp;
