/*	NEWCASCADE(DTA)
	This program is currently configured to use only (the demeaned) absolute values in the classifying equation, instead of the quadratic form.
*/


proc (1)= newcascade(dta);
local itertime, nobs, swd, beta0, xols, g, h, retcode, waldstat, yjunk,
	llfvr, llfvrp, llfec, llfecp, llfnm, llfnmp, llfsw, llfswp, 
	xvr, xvrp, xec, xecp, xnm, xnmp, xsw, xswp, 
	fit, pclass, bullbear, testmean;

"SWCascade - Robust Estimation of Nested Switching Models";
itertime = date;
nobs=rows(dta);

_emprint=0;

@ The VR Model @
__title="Volatility Regimes Model: (Constant Switching Prob.)";
?;      __title;        
_emecm=1;
swd = dta[.,1]~ones(nobs,2);
_n1=1;  _n2=1;  _n3=1;
xols=swd[.,2];		{beta0} = becinit(swd);
_max_ParNames = "B_0  " | "B_q  " | "S_S  " | "S_C  ";
{xvr,llfvr,g,h,retcode}=swemmle(beta0,swd);
maxrprt(_max_ParNames,xvr,h,_max_IterData,retcode);

@ The VRP Model @
__title="Volatility Regimes Model: (Variable Switching Prob.)";
?;      __title;        
_emecm=1;
/*
swd = dta[.,1]~ones(nobs,2)~dta[.,2]~(dta[.,2].^2);
_n1=1;  _n2=1;  _n3=3;
beta0 = xvr[1 2,1] | zeros(2,1) | xvr[3 4,1];
_max_ParNames = "B_0  " | "B_q  "  | "B_qb " | "B_qb2" | "S_S  " | "S_C  ";
*/
swd = dta[.,1]~ones(nobs,2)~(abs(dta[.,2])-meanc(abs(dta[.,2])));
_n1=1;  _n2=1;  _n3=2;
beta0 = xvr[1 2,1] | zeros(1,1) | xvr[3 4,1];
_max_ParNames = "B_0  " | "B_q  "  | "B_qb " | "S_S  " | "S_C  ";
{xvrp,llfvrp,g,h,retcode}=swemmle(beta0,swd);
maxrprt(_max_ParNames,xvrp,h,_max_IterData,retcode);

@ The EC Model @
__title="Error Correction Model: (Constant Switching Prob.)";
?;      __title;        
_emecm=1;
swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1);
_n1=2;  _n2=1;  _n3=1;
xols=swd[.,2:3];		{beta0} = becinit(swd);
@beta0 = xvr[1,1] | zeros(1,1) | xvr[2:4,1];@
_max_ParNames = "B_0  " | "B_b  " | "B_q  " | "S_S  " | "S_C  ";
{xec,llfec,g,h,retcode}=swemmle(beta0,swd);
maxrprt(_max_ParNames,xec,h,_max_IterData,retcode);

@ The ECP Model @
__title="Error Correction Model: (Variable Switching Prob.)";
?;      __title;        
_emecm=1;
/*
swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~(dta[.,2].^2);
_n1=2;  _n2=1;  _n3=3;
beta0 = xec[1:3,1] | zeros(2,1) | xec[4:5,1];
_max_ParNames = "B_S0 " | "B_C0 " | "B_q  " | "B_qb " | "B_qb2" | "S_S  " | "S_C  ";
*/
swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~(abs(dta[.,2])-meanc(abs(dta[.,2])));
_n1=2;  _n2=1;  _n3=2;
beta0 = xec[1:3,1] | zeros(1,1) | xec[4:5,1];
_max_ParNames = "B_S0 " | "B_C0 " | "B_q  " | "B_qb " | "S_S  " | "S_C  ";
{xecp,llfecp,g,h,retcode}=swemmle(beta0,swd);
maxrprt(_max_ParNames,xecp,h,_max_IterData,retcode);

@ The NM Model @
__title="Mixture of Normal Distributions Model: (Constant Switching Prob.)";
?;      __title;        
_emecm=0;
swd = dta[.,1]~ones(nobs,3);
_n1=1;  _n2=1;  _n3=1;
{beta0}=binitial(swd);
@beta0 = xvr[1,1] | xvr[1,1] | xvr[2:4,1];@
_max_ParNames = "B_S0 " | "B_C0 " | "B_q  " | "S_S  " | "S_C  ";
{xnm,llfnm,g,h,retcode}=swemmle(beta0,swd);
maxrprt(_max_ParNames,xnm,h,_max_IterData,retcode);

@ The NMP Model @
__title="Mixture of Normal Distributions Model: (Variable Switching Prob.)";
?;      __title;        
_emecm=0;
/*
swd = dta[.,1]~ones(nobs,3)~dta[.,2]~(dta[.,2].^2);
_n1=1;  _n2=1;  _n3=3;
beta0 = xnm[1:3,1] | zeros(2,1) | xnm[4 5,1];
_max_ParNames = "B_S0 " | "B_C0 " | "B_q  " | "B_qb " | "B_qb2" | "S_S  " | "S_C  ";
*/
swd = dta[.,1]~ones(nobs,3)~(abs(dta[.,2])-meanc(abs(dta[.,2])));
_n1=1;  _n2=1;  _n3=2;
beta0 = xnm[1:3,1] | zeros(1,1) | xnm[4 5,1];
_max_ParNames = "B_S0 " | "B_C0 " | "B_q  " | "B_qb " | "S_S  " | "S_C  ";
{xnmp,llfnmp,g,h,retcode}=swemmle(beta0,swd);
maxrprt(_max_ParNames,xnmp,h,_max_IterData,retcode);

@ The SW Model @
__title="Switching Regression Model: (Constant Switching Prob.)";
?;      __title;        
_emecm=0;
swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~ones(nobs,1);
_n1=2;  _n2=2;  _n3=1;
if llfnm > llfec;
	beta0 = xnm[1,1] |zeros(1,1)| xnm[2,1] |zeros(1,1)| xnm[3:5,1];
else;
	beta0 = xec[1:2,1] | xec[1:2,1] | xec[3:5,1];
endif;
_max_ParNames = "B_S0 " | "B_Sb " | "B_C0 " | "B_Cb " | "B_q  " | "S_S  " | "S_C  ";
{xsw,llfsw,g,h,retcode}=swemmle(beta0,swd);
maxrprt(_max_ParNames,xsw,h,_max_IterData,retcode);

@ The SWP Model @
_emprint=0;
__title="Switching Regression Model: (Variable Switching Prob.)";
?;      __title;        
_emecm=0;
/*
swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~(dta[.,2].^2);
_n1=2;  _n2=2;  _n3=3;
if     (llfsw > llfecp);
    beta0 = xsw[1:5,1] | zeros(2,1) | xsw[6:7,1];
else;
    beta0 = xecp[1:2,1] | xecp[1:2,1] | xecp[3:7,1];
endif;
_max_ParNames = "B_S0 " | "B_Sb "  | "B_C0 " | "B_Cb " | "B_q  " | "B_qb " | "B_qb2" | "S_S  " | "S_C  ";
*/
swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~(abs(dta[.,2])-meanc(abs(dta[.,2])));
_n1=2;  _n2=2;  _n3=2;
if     (llfsw > llfecp);
    beta0 = xsw[1:5,1] | zeros(1,1) | xsw[6:7,1];
else;
    beta0 = xecp[1:2,1] | xecp[1:2,1] | xecp[3:6,1];
endif;
_max_ParNames = "B_S0 " | "B_Sb "  | "B_C0 " | "B_Cb " | "B_q  " | "B_qb " | "S_S  " | "S_C  ";
{xswp,llfswp,g,h,retcode}=swemmle(beta0,swd);

"-----------------------------------------------------------------------------";
"Likelihood Ratio Tests:";
llfvr  = llfvr *nobs;	"VR:  llf=";;	llfvr;;		", # parameters = 4";
llfvrp = llfvrp*nobs;	"VRP: llf=";;	llfvrp;;	", # parameters = 5";
llfec  = llfec *nobs;	"EC:  llf=";;	llfec;;		", # parameters = 5";
llfecp = llfecp*nobs;	"ECP: llf=";;	llfecp;;	", # parameters = 6";
llfnm  = llfnm *nobs;	"NM:  llf=";;	llfnm;;		", # parameters = 5";
llfnmp = llfnmp*nobs;	"NMP: llf=";;	llfnmp;;	", # parameters = 6";
llfsw  = llfsw *nobs;	"SW:  llf=";;	llfsw;;		", # parameters = 7";
llfswp = llfswp*nobs;	"SWP: llf=";;	llfswp;;	", # parameters = 8";
"Test of VR  vs SWP : ";;  2*(llfswp -llfvr );;
"   Signif. = ";;  cdfchic(2*(llfswp -llfvr ),4);
"Test of VRP vs SWP : ";;  2*(llfswp -llfvrp);;
"   Signif. = ";;  cdfchic(2*(llfswp -llfvrp),3);
"Test of EC  vs SWP : ";;  2*(llfswp -llfec );;
"   Signif. = ";;  cdfchic(2*(llfswp -llfec ),3);
"Test of ECP vs SWP : ";;  2*(llfswp -llfecp);;
"   Signif. = ";;  cdfchic(2*(llfswp -llfecp),2);
"Test of NM  vs SWP : ";;  2*(llfswp -llfnm );;
"   Signif. = ";;  cdfchic(2*(llfswp -llfnm ),3);
"Test of NMP vs SWP : ";;  2*(llfswp -llfnmp);;
"   Signif. = ";;  cdfchic(2*(llfswp -llfnmp),2);
"Test of SW  vs SWP : ";;  2*(llfswp -llfsw );;
"   Signif. = ";;  cdfchic(2*(llfswp -llfsw ),1);

yjunk = 1 ~ -1;
"Wald Test for Beta_S0 = Beta_C0:   is Chi-sq(1) under the null.";
waldstat = yjunk*h[1 _n1+1,1 _n1+1]*yjunk' ;
waldstat = (xswp[1,1] - xswp[_n1+1,1])^2 ./ waldstat;
waldstat[1,1];;      "   Signif. = ";; cdfchic(waldstat[1,1],1);

"Wald Test for Beta_Sb = Beta_Cb:   is Chi-sq(1) under the null.";
waldstat = yjunk*h[2 _n1+2,2 _n1+2]*yjunk' ;
waldstat = (xswp[2,1] - xswp[_n1+2,1])^2 ./ waldstat;
waldstat[1,1];;      "   Signif. = ";; cdfchic(waldstat[1,1],1);

swmisspc(xswp,swd);

"Total Estimation time = ";;  etstr(ethsec(itertime,date));
"=============================================================================";
?; ?;
retp(xswp);

endp;
