/*
This file contains the likelihood and gradient procs for use in
error contamination models with maxlik.
Note that they relax the assumption that the same regressors are used in all 3
regression equations.  Therefore, THE USER MUST SET _N1 _N2 and _N3 in the
main program so that li() knows which parameters belong in which equation.
(I use all 3 variables to maintain consistency with swli_new.g.  Actually, only
_n1 and _n3 are used.)
*/

clearg _n1, _n2, _n3;

proc swecli(b,dta);
local bigphi, LITLPHI1, LITLPHI2, LF, res1, res2, phi1, phi2, phi3,
      y1, x1, x2, x3, betas1, betas2, betas3, sigma1, sigma2;

@ sigma3 is assumed to be 1. @

@  Retrieve the current parameter values.  @
    betas1   = b[1:_n1,1];
    betas3   = b[(_n1+1):(_n1+_n3),1];
    sigma1   = b[(_n1+_n3+1),1];
    sigma2   = b[(_n1+_n3+2),1];
    y1 = dta[.,1];
    x1  = dta[.,2:(_n1+1)];
    x3  = dta[.,(_n1+2):(_n1+_n3+1)];


    x2 = y1-x1*betas1;
    phi1    = pdfn(x2./abs(sigma1))./abs(sigma1);
    phi2    = pdfn(x2./abs(sigma2))./abs(sigma2);
    phi3    = cdfn(x3*betas3);


    phi1 = ln((1-phi3) .* phi1 + phi3 .* phi2);

    @ This allows for underflows in the above expression. @
    if ndpchk(3);   ndpclex;    endif; 

retp( phi1 );
endp;


proc swecgrad(b,dta);
local  weit1, weit2, res1, res2, phi1, phi2, phi3, n,
       y1, x1, x2, x3, betas1, betas2, betas3, sigma1, sigma2;

@ We assume that sigma3 = 1.0 for simplicity.  @

@  Retrieve the current parameter values.  @
    betas1   = b[1:_n1,1];
    betas3   = b[(_n1+1):(_n1+_n3),1];
    sigma1   = b[(_n1+_n3+1),1];
    sigma2   = b[(_n1+_n3+2),1];
    y1 = dta[.,1];
    x1  = dta[.,2:(_n1+1)];
    x3  = dta[.,(_n1+2):(_n1+_n3+1)];

    x2 = y1-x1*betas1;
    phi1    = pdfn(x2./abs(sigma1))./abs(sigma1);
    phi2    = pdfn(x2./abs(sigma2))./abs(sigma2);
    phi3    = cdfn((x3*betas3));

    weit1   = (1-phi3).*phi1;
    weit2   = weit1 + phi3.*phi2;
    @ The following allows for underflows in the above expression. @
    if ndpchk(3);
        weit1 = (weit1 + 1) - 1;
        ndpclex;
    endif;

    res1    = x2./abs(sigma1);
    res2    = x2./abs(sigma2);
    weit1   = weit1./weit2;

    retp(   (weit1.*res1.*x1)./abs(sigma1)+((1-weit1).*res2.*x1)./abs(sigma2) ~
            (pdfn(-x3*betas3)./(phi1./(phi2-phi1)+phi3)).*x3 ~
            -weit1.*(1 - res1.*res1)./abs(sigma1) ~
            (weit1-1).*(1 - res2.*res2)./abs(sigma2)        );

endp;


