/*  This file contains a procedure swem() that takes a parameter vector and a data matrix, and then runs once through the EM estimation algorithm for switching regressions described by Hartley(1979), and returns an updated estimate of the parameter vector.  The conventions for the form of the data matrix and the parameter vector are same as for swli(), described in swli_new.g.   */

clearg _n1, _n2, _n3, nobs;

proc swem(binitial,dta);
local f1, f2, h, lamda, w1, w2, epsilon, beta1, beta2, beta3, t1, t2, sigma1, sigma2, nparam, y, x1, x2, x3, y1, y2;

beta1 = binitial[1:_n1,1];
beta2 = binitial[(_n1+1):(_n1+_n2),1];
beta3 = binitial[(_n1+_n2+1):(_n1+_n2+_n3),1];
sigma1 = binitial[(_n1+_n2+_n3+1),1];
sigma2 = binitial[(_n1+_n2+_n3+2),1];

y=dta[.,1];
x1=dta[.,2:(_n1+1)];
x2=dta[.,(_n1+2):(_n1+_n2+1)];
x3=dta[.,(_n1+_n2+2):(_n1+_n2+_n3+1)];

f1=pdfn((y-x1*beta1)./sigma1)./sigma1;
f2=pdfn((y-x2*beta2)./sigma2)./sigma2;
lamda=cdfn(-x3*beta3);
h=(lamda.*f1)+((1-lamda).*f2);
w1=lamda.*f1./h;
w2=1-w1;
epsilon=x3*beta3+(f2-f1).*pdfn(-x3*beta3)./h;

t1=sumc(w1);
t2=sumc(w2);
w1 = sqrt(w1);
w2 = sqrt(w2);

@"Ave. weights: w1=";; t1/nobs;; " w2=";; t2/nobs;@

x1=x1.*w1;
x2=x2.*w2;
y1=y.*w1;
y2=y.*w2;
beta1=y1/x1;
beta2=y2/x2;
beta3=epsilon/x3;

sigma1=y1-x1*beta1;
sigma2=y2-x2*beta2;
sigma1=sqrt(sigma1'*sigma1/t1);
sigma2=sqrt(sigma2'*sigma2/t2);


retp(beta1|beta2|beta3|sigma1|sigma2);

endp;
