@clearg _emmaxit, _emconvg, _emecm, _emfull;@
/*
@ Set Defaults @
maxset;
_emmaxit=101;	@ Maximum number of EM iterations @
_emconvg=0.05;	@ EM Convergence criterion @
_emecm=0;	@ >0 will use swecmem, otherwise swem @
_emprint=0;	@ >0 gives maxlik output using maxprt, <0 prints each EM iteration @
*/

proc (5) = swemmle(b0,dta);
local newparam, oldparam, nparam, convcrit, iter, itertime;

if rows(b0)/=cols(dta)+1;
	"ERROR: Dimensions of parameter vector and dataset do not match."
	"There are ";; rows(b0);; " parameters and ";; cols(dta);; " variables.";
	retp(0,0,0,0,101);
else;
	@ The EM algorithm @
	oldparam=b0;
	convcrit = 1;
	iter = 1;
	nparam = rows(oldparam);
	itertime = date;
	do while convcrit>_emconvg and iter<=_emmaxit;
		if _emecm>0;
			newparam = swecmem(oldparam,dta);
		else;
			newparam = swem(oldparam,dta);
		endif;
		convcrit=(oldparam-newparam)./newparam;
@		convcrit=sqrt(convcrit'*convcrit/nparam);@
		convcrit=sqrt(maxc(convcrit.*convcrit));
		if _emprint<0;
			"Iteration #";; iter;;
			"	Convergence Criterion = ";;convcrit;
			"new parameters:";; newparam';
		endif;
		oldparam=newparam;
		iter=iter+1;
	endo;
	itertime=ethsec(itertime,date);
	if _emprint>0;
		if iter>_emmaxit;
			"EM algorithm iteration limit exceeded after ";;
			_emmaxit;; " iterations, Convergence Criterion = ";;convcrit;
		else;
			"EM algorithm converged in " etstr(itertime);;
			", ";;	iter-1;; " iterations.";
		endif;
		if __output==0;
			"EM estimates:";; newparam';
		endif;
	endif;
	@ The MAXLIK algorithm @
	if _emecm>0;
		_max_GradProc=&swecgrad;
		if _emprint>0;
			retp(maxprt(maxlik(dta,0,&swecli,newparam)));
		else;
			retp(maxlik(dta,0,&swecli,newparam));
		endif;
	else;
		_max_GradProc=&swgrad;
		if _emprint>0;
			retp(maxprt(maxlik(dta,0,&swli,newparam)));
		else;
			retp(maxlik(dta,0,&swli,newparam));
		endif;
	endif;
endif;

endp;
