/*  This proc computes starting values for switching regressions or normal
mixtures.

It is assumed that the first column of the dataset contains the dependent
(switching) variable. Note that the regressors need not be the same in all 3
regressions and a constant vector is not added.  The global variables _n1, _n2
and _n3 indicate the size of the parameter vectors for each equation. Rows in
the data set with missing values are ignored using "packr".
  
The proc returns a vector of inital values for use in MAXLIK.  Positive values
of y are associated with equation 1.  */


@ First, define selif. @
proc selif(x,e);
    retp( submat(x,submat(packr(seqa(1,1,rows(x))~miss(e,0)),0,1),0) );
endp;





proc 1=binitial(initd);
local b1, b2, b3, sigma1, sigma2, selctval;

initd = packr(initd);

selctval = selif(initd[.,1:_n1+1],initd[.,1].>=0);
b1 = selctval[.,1] / selctval[.,2:_n1+1];
sigma1 = stdc( selctval[.,1] - selctval[.,2:_n1+1]*b1 );

selctval = selif(initd[.,1 _n1+2:_n1+_n2+1],initd[.,1].<0);
b2 = selctval[.,1] / selctval[.,2:_n2+1];
sigma2 = stdc( selctval[.,1] - selctval[.,2:1+_n2]*b2 );

selctval = initd[.,1] - meanc(initd[.,1]);
selctval = cdfn( selctval./stdc(selctval[.,1]) );
b3 = selctval[.,1] / initd[.,_n1+_n2+2:_n1+_n2+_n3+1];

retp(b1|b2|b3|sigma1|sigma2);
endp;
